/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2018 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 5.00a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_CY8C5868.c for Cypress PSoC CY8C5868
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency   */
  #define OS_FSYS  (50000000u)
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer */
  #define OS_PCLK_TIMER  (OS_FSYS)  /* May vary from CPU clock    */
#endif                              /* depending on CPU           */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART  */
  #define OS_PCLK_UART   (OS_FSYS)  /* May vary from CPU clock    */
#endif                              /* depending on CPU           */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ   (1000u)
#endif

#define OS_TIMER_RELOAD  (OS_PCLK_TIMER / OS_TICK_FREQ)

#ifndef   OS_USE_VARINTTABLE        /* The interrupt vector table   */
  #define OS_USE_VARINTTABLE (1)    /* has to be located in RAM     */
#endif

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables OS_UART 0 (UART0) per
*       default and supports UART0 only.
*       To use the UART you have to connect UART Rx and Tx
*       to the appropriate pins with cable jumpers.
*       UART | Pin
*       =====|=====
*       Rx   | P3_4
*       Tx   | P3_3
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #ifndef   OS_UART
    #define OS_UART     (0)
  #endif

  #ifndef   OS_BAUDRATE
    #define OS_BAUDRATE (38400u)
  #endif
#endif

/****** End of configurable options *********************************/

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/

/****** System control **********************************************/
#define SYS_INT_CTRL_STATE        (*(volatile OS_U32*)(0xE000ED04u))
#define SYS_PENDSTSET             (26u)

/****** The system timer ********************************************/
#define SYS_TICK_BASE_ADDR        (0xE000E010u)
#define SYS_TICK_CONTROL          (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x00u))
#define SYS_TICK_RELOAD           (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x04u))
#define SYS_TICK_VALUE            (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x08u))
#define SYS_TICK_ENABLE_BIT       (0u)
#define SYS_TICK_CLK_SOURCE_BIT   (2u)

/****** NVIC, vectored interrupt controller *************************/
#define NUM_INTERRUPTS            (16u + 32u)

/*****  Interrupt ID numbers ****************************************/
#define ISR_ID_SYSTICK            (15u)               // System Tick
#define ISR_ID_OS_EXCEPTION       (14u)               // PendSV

#define ISR_ID_VIEW_RX            (17u)               // View Rx
#define ISR_ID_VIEW_TX            (18u)               // View Tx
#define ISR_VIEW_PRIO             (0xC0u)             // View Rx/Tx interrupt priority

#define ISR_OS_EXCEPTION_PRIO     (0xFFu)             // Lowest priority. Required !

/*********************************************************************
*
*       Extern declarations
*
**********************************************************************
*/
extern void* RomVectors;
extern void* CyRamVectors;

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKMEM.h"
  const OS_U32 OS_JLINKMEM_BufferSize = 32u;  // Size of the communication buffer for JLINKMEM
#else
  const OS_U32 OS_JLINKMEM_BufferSize = 0;    // Communication not used
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value
*
*  Return value
*    Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return SYS_TICK_VALUE;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set
*
*  Return value
*    == 0; Interrupt pending flag not set
*    != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return SYS_INT_CTRL_STATE & (1u << SYS_PENDSTSET);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the code that gets called when the processor receives a
*    _SysTick exception. SysTick is used as OS timer tick.
*
*  Notes:
*    (1) Must be inserted into the interrupt vector table if RAM
*        vectors are not used. Hence it is declared public.
*/
#ifdef __cplusplus
extern "C" {
#endif
void SysTick_Handler(void);
#ifdef __cplusplus
}
#endif
void SysTick_Handler(void) {
  OS_INT_EnterNestable();
  OS_TICK_Handle();
  #if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
    JLINKMEM_Process();
  #endif
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware (timer) required for embOS to run.
*    May be modified, if an other timer should be used.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_FSYS, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
void OS_InitHW(void) {
  int TickPrio;

  OS_INT_IncDI();
  //
  // Initialize OS timer, clock soure = core clock
  //
  SYS_TICK_RELOAD  = OS_TIMER_RELOAD;
  SYS_TICK_CONTROL = (1 << SYS_TICK_ENABLE_BIT) | (1 << SYS_TICK_CLK_SOURCE_BIT);
  //
  // Install Systick Timer Handler with second lowest priority and enable timer interrupt
  //
  OS_ARM_ISRInit(OS_USE_VARINTTABLE, NUM_INTERRUPTS, (OS_ISR_HANDLER**)&RomVectors, (OS_ISR_HANDLER**)&CyRamVectors);
  OS_ARM_InstallISRHandler(ISR_ID_SYSTICK, (OS_ISR_HANDLER*)SysTick_Handler);
  OS_ARM_ISRSetPrio(ISR_ID_SYSTICK, 0xFF);               // Set to lowest level, ALL BITS set
  TickPrio  = OS_ARM_ISRSetPrio(ISR_ID_SYSTICK, 0xFF);   // Read priority back to examine relevant preemption-level-bits
  TickPrio -= 1;                                         // Set to low preemption level, 1 level higher than lowest
  OS_ARM_ISRSetPrio(ISR_ID_SYSTICK, TickPrio);
  OS_ARM_EnableISR(ISR_ID_SYSTICK);
  //
  // Install embOS scheduler interrupt with lowest priority and enable it
  //
  OS_ARM_InstallISRHandler(ISR_ID_OS_EXCEPTION, (OS_ISR_HANDLER*)PendSV_Handler);
  OS_ARM_ISRSetPrio(ISR_ID_OS_EXCEPTION, ISR_OS_EXCEPTION_PRIO);
  OS_ARM_EnableISR(ISR_ID_OS_EXCEPTION);
  //
  // Setup values for usec precise system time functions
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Configure and initialize SEGGER SystemView
  //
#if OS_PROFILE
  SEGGER_SYSVIEW_Conf();
#endif
  //
  // Initialize the optional communication for embOSView
  //
#if (OS_VIEW_IFSELECT != OS_VIEW_DISABLED)
  OS_COM_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This is basically the "core" of the idle loop.
*    This core loop can be changed, but:
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved. However, a simple program loop can be programmed
*    (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
      #ifdef __ICCARM__  // IAR
        __asm(" wfi");
      #endif
      #ifdef __CC_ARM    // KEIL
        __wfi();
      #endif
      #ifdef __GNUC__    // GCC
        __asm(" wfi");
      #endif
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  Function description
*    This routine is required for task-info via embOSView or high
*    resolution time measurement functions.
*    It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_TIME_GetTicks32();
  Cnt  = OS_TIMER_RELOAD - SYS_TICK_VALUE;
  //
  // Check if timer interrupt pending ...
  //
  if (SYS_INT_CTRL_STATE & (1u << SYS_PENDSTSET)) {
    Cnt = OS_TIMER_RELOAD - SYS_TICK_VALUE;  // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Function description
*    Convert Cycles into micro seconds.
*
*    If your clock frequency is not a multiple of 1 MHz,
*    you may have to modify this routine in order to get proper
*    diagnostics.
*
*    This routine is required for profiling or high resolution time
*    measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return (Cycles / (OS_TIMER_RELOAD / 1000u));
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character via J-Link
*/
void OS_COM_Send1(OS_U8 c) {
  JLINKMEM_SendChar(c);
}

/*********************************************************************
*
*       OS_COM_Init()
*
*  Function description
*    Initializes J-Link communication for embOSView
*/
void OS_COM_Init(void) {
  JLINKMEM_SetpfOnRx(OS_COM_OnRx);
  JLINKMEM_SetpfOnTx(OS_COM_OnTx);
  JLINKMEM_SetpfGetNextChar(OS_COM_GetNextChar);
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)

#include "View_view.h"

/*********************************************************************
*
*       OS_COM_IsrHandler()
*
*  Function description
*    The communication interrupt handler for UART communication
*    to embOSView.
*
*  Notes:
*    (1) Not required here, separate file.
*/

/*********************************************************************
*
*       OS_COM_Init()
*
*  Function description
*    Initializes Uart communication for embOSView
*/
void OS_COM_Init(void) {
  //
  // Initialize and start UART
  // enable UART interrupts
  View_Start();
  //
  // Set interrupt priority for Tx/Rx interrupts to
  // a higher value as in the generated sources.
  // This is necessary for a working step tick mode in
  // embOSView.
  //
  OS_ARM_ISRSetPrio(ISR_ID_VIEW_RX, ISR_VIEW_PRIO);
  OS_ARM_ISRSetPrio(ISR_ID_VIEW_TX, ISR_VIEW_PRIO);
}

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character via UART
*/
void OS_COM_Send1(OS_U8 c) {
  View_WriteData(c);
}

#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Dummy routine.
*/
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
}

#else
  #error "Selected embOSView interface is currently not supported."
#endif

/****** End Of File *************************************************/
